﻿# -*- coding: utf-8 -*-
####################################################################
#	APPLICATION FORM FOR STOCK ON SOCIAL HELP MANAGEMENT, CHILE.
#		Created and Designed by:
#			David Acevedo (dacevedo@stratanet.cl)
#			Diego Cantos (dcantos@stratanet.cl)
#	 	www.stratanet.cl
####################################################################

from osv import osv, fields
import datetime, addons
from tools.translate import _

class social_help_voucher_item(osv.osv):
	_name = 'social.help.voucher.item'
	_description = 'Social Report Item Reference'
	_rec_name = 'table_id'
	
	_columns = {
		'table_id':fields.integer('Id', required=True, readonly=False),
		'company_id':fields.many2one('res.company', 'Name', required=True, readonly=False),
		'item_id':fields.many2one('social.help.stock.items', 'Item Id', required=True, readonly=False),
		'amount':fields.integer('Requested Amount', required=True, readonly=False)
	}
	
	def create(self, cr, user, vals, context=None):
		if 'item_id' in vals:
			amount = vals['amount']
			item_id = vals['item_id']

			virtual_amount = self.pool.get('social.help.stock.items').browse(cr, user, item_id, context=context).virtual_amount
			virtual_amount -= amount

			if virtual_amount < 0:
				raise osv.except_osv(_('Error!'),_('You do not have the number of items requested to be delivered'))
			else:
				self.pool.get('social.help.stock.items').write(cr, user, item_id, {'virtual_amount': virtual_amount})

			res = super(social_help_voucher_item, self).create(cr, user, vals, context=context)
			return res	
		else:
			res = super(social_help_voucher_item, self).create(cr, user, vals, context=context)
			return res	

	_defaults = {
		'company_id': lambda self,cr,uid,c: self.pool.get('res.company')._company_default_get(cr, uid,\
			'social.help.voucher.item',context=c),
	}
social_help_voucher_item()

class social_sector(osv.osv):
	_name = 'social.sector'
	_descripcion = 'Sectors of the Place'

	_columns = {
		'name':fields.char('Sector Name', size=64, required=True),
		'code':fields.char('Sector Code', size=5, required=True),
		'count':fields.char('Amount of helps', size=20, required=True, readonly=False)
	}

	def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=80):
		if not args:
			args = []
		if not context:
			context = {}
		ids = self.search(cr, user, [('code', operator, name)] + args, limit=limit, context=context)
		
		if not ids:
			ids = self.search(cr, user, [('name', operator, name)] + args, limit=limit, context=context)
		return self.name_get(cr, user, ids, context)

	_defaults = {
		'count': '0'
	}
social_sector()

class social_family_group(osv.osv):
	_name = 'social.family.group'
	_descripcion = 'Family Group'
	_rec_name = 'family_folio'
	
	_columns = {
		'family_folio':fields.char('Unic National Folio', size=30, required=True, readonly=False),
		'family_jjvv':fields.char('JJ.VV', size=60, required=False, readonly=False),
		'family_sector':fields.many2one('social.sector', 'Sector', required=True, readonly=False),
		'family_score':fields.char('Beneficiary Score', size=25, required=False, readonly=False),
		'family_coord':fields.char('Geographical Coordinates', size=10, required=False, readonly=False),
		'family_total_income':fields.integer('Total Income', size=10, required=False, readonly=False),
		'family_percapita_income':fields.integer('Percapita Income', size=10, required=False, readonly=False),
		'family_dwelling':fields.text('Beneficiary Dwelling', required=False, readonly=False),
		'family_income_description':fields.text('Income', required=False, readonly=False),
		'family_num_men':fields.integer('Men', required=False, readonly=False),
		'family_num_women':fields.integer('Women', required=False, readonly=False),
		'family_num_children':fields.integer('Children', required=False, readonly=False),
		'family_total_num':fields.integer('Total', required=False, readonly=False),
	}
social_family_group()

class social_family_group_member(osv.osv):
	_name = 'social.family.group.member'
	_description = 'Family Members'
	_rec_name = 'member_rut'
	
	_columns = {
		'member_rut':fields.char('RUT', size=32, readonly=False, required=True),
		'member_folio':fields.many2one('social.family.group','Unic National Folio', required=True, readonly=False),
		'member_full_name':fields.char('Full Name', size=32, readonly=False, required=True),
		'member_gender':fields.selection((('masculino','Male'),('femenino','Female')), 'Gender', required=True, readonly=False),
		'member_age':fields.integer('Age', required=True, readonly=False),
		'member_date_birth':fields.date('Date of Birth', required=True, readonly=False),
		'member_address':fields.char('Address', size=60, required=True, readonly=False),
		'member_phone':fields.char('Phone Number', size=25, required=True, readonly=False),
		'member_kinship':fields.char('Kinship', size=15, required=False, readonly=False),
		'member_marital_status':fields.selection((
			('married','Married'),\
			('single','Single'),\
			('divorced','Divorced'),\
			('widow','Widow/er'),\
			('cohabitant','Cohabitant')), 'Marital Status', required=True, readonly=False),
		'member_schooling':fields.selection((
			('primary_com','Primary School Complete'),\
			('primary_unc','Primary School Uncomplete'),\
			('secondary_com','High School Complete'),\
			('secondary_unc','High School Uncomplete'),\
			('college_com','College Complete'),\
			('college_unc','College Uncomplete'),\
			('technical_com','Technical Degree Complete'),\
			('technical_unc','Technical Degree Uncomplete'),\
			('none','None')), 'Schooling', required=True, readonly=False),
		'member_activity':fields.char('Current Activity', size=25, required=False, readonly=False),
		'member_income':fields.integer('Income', size=10, required=True, readonly=False),
		'member_health':fields.text('Beneficiary Health', required=False, readonly=False),
		'member_health_insurance':fields.selection((
			('fonasa','FONASA'),\
			('isapre','ISAPRE'),\
			('none','None')), 'Health Insurance', required=True, readonly=False),
		'member_suf':fields.char('SUF', size=25, required=False, readonly=False), #Subsidio Unico Familiar
		'member_pension':fields.selection((
			('afp','AFP'),
			('state','State'),\
			('isp','ISP'),\
			('none','None')), 'Pension Assistance', required=True, readonly=False),
		'member_situation_eva':fields.text('Situation and Professional Evaluation', required=False, readonly=False),
		'member_observations':fields.text('Observations', required=False, readonly=False),

	}

	def check_run(self, body, vdig):
		try:
			operar = (range(10) + ['k'])[11-sum([int(digit)*factor for digit,factor in zip(body[::-1],2*range(2,8))])%11]
			try:
				if operar == int(vdig):
					return True
				else:
					return False
			except ValueError:
				if operar == vdig.lower():
					return True
				else:
					return False
		except IndexError:
			return False

	def _check(self, cr, uid, ids):
		obj = self.browse(cr, uid, ids)
		
		for record in obj:
			if record.member_rut:
				try:
					body, vdig = record.member_rut.split('-')
					if not self.check_run(body, vdig):
						raise osv.except_osv(_('Error!'),_("Insert a valid RUT in the field 'RUT'"))
				except ValueError:
					raise osv.except_osv(_('Error!'),_("Insert a valid RUT in the field 'RUT'"))

				return True

	_sql_constraints = [
		('rut_uniq', 'unique (member_rut)', _('The Rut is already in the records!')),
	]

	_constraints = [
		(_check, '', ['']),
	]
social_family_group_member()	


class social_help_voucher(osv.osv):
	_name = 'social.help.voucher'
	_description = 'Social Report Form'
	_rec_name = 'voucher_code'

	def _search_family_group(self, cr, uid, ids, field_name, arg, context=None):
		"""
		@param cr: the current row, from the database cursor,
		@param uid: the current user’s ID for security checks,
		@param ids: List of create menu’s IDs
		@param context: A standard dictionary for contextual values """
	
		res = {}

		for id in ids:
			res[id] = {
				'family_jjvv': False,
				'family_score': False,
				'family_sector': False,
				'family_total_income': False,
				'family_percapita_income': False,
				'family_dwelling': False,
				'family_income_description': False,
				'family_num_men': False,
				'family_num_women': False,
				'family_num_children': False,
				'family_total_num': False
			}

		if not ids:
			return res	
		else:
			cr.execute("SELECT v.id, g.family_jjvv, g.family_score, s.name, g.family_total_income, g.family_percapita_income, \
					g.family_dwelling, g.family_income_description, g.family_num_men, g.family_num_women, \
					g.family_num_children, g.family_total_num from social_family_group as g \
					inner join social_help_voucher as v on (g.id = v.family_id) \
					inner join social_sector as s on (s.id = g.family_sector) \
					where v.id IN %s \
					order by v.id;",(tuple(ids),))

			for id, family_jjvv, family_score, name, family_total_income, family_percapita_income, family_dwelling,\
				family_income_description, family_num_men, family_num_women, family_num_children,\
				family_total_num in cr.fetchall():
				res[id]['family_jjvv'] = family_jjvv
				res[id]['family_score'] = family_score
				res[id]['family_sector'] = name
				res[id]['family_total_income'] = family_total_income
				res[id]['family_percapita_income'] = family_percapita_income
				res[id]['family_dwelling'] = family_dwelling
				res[id]['family_income_description'] = family_income_description
				res[id]['family_num_men'] = family_num_men
				res[id]['family_num_women'] = family_num_women
				res[id]['family_num_children'] = family_num_children
				res[id]['family_total_num'] = family_total_num
			return res

	def _search_family_member(self, cr, uid, ids, field_name, arg, context=None):
		"""
		@param cr: the current row, from the database cursor,
		@param uid: the current user’s ID for security checks,
		@param ids: List of create menu’s IDs
		@param context: A standard dictionary for contextual values """
	
		res = {}

		for id in ids:
			res[id] = {
				'member_full_name': False,
				'member_address': False,
				'member_phone': False,
				'member_health': False,
				'member_situation_eva': False,
				'member_age': False,
			}

		if not ids:
			return res	
		else:
			cr.execute("SELECT v.id, m.member_full_name, m.member_address, m.member_phone, m.member_health, \
					m.member_situation_eva, m.member_age from social_family_group_member as m \
					inner join social_help_voucher as v on (m.id = v.member_id) \
					where v.id IN %s \
					order by v.id;",(tuple(ids),))

			for id, member_full_name, member_address, member_phone, member_health, member_situation_eva, member_age in cr.fetchall():
				res[id]['member_full_name'] = member_full_name
				res[id]['member_address'] = member_address
				res[id]['member_phone'] = member_phone
				res[id]['member_health'] = member_health
				res[id]['member_situation_eva'] = member_situation_eva
				res[id]['member_age'] = member_age
			return res

	_columns = {
		'company_id':fields.many2one('res.company', 'Company', required=False, readonly=False),
		'voucher_code':fields.integer('Social Report Code', readonly=True, required=True),
		'statement':fields.integer('Statement Code', readonly=False, required=True,\
			help="0 if the social help don't have statement "),
		'family_id':fields.many2one('social.family.group','Search Unic National Folio', required=True, readonly=False),
		'family_jjvv':fields.function(_search_family_group, method=True, type='char', size=60, string='JJ.VV', multi='family_group'),
		'family_score':fields.function(_search_family_group, method=True, type='char', size=25, string='Beneficiary Score',\
			multi='family_group'),
		'family_sector':fields.function(_search_family_group, method=True, type='char', size=64, string='Sector', multi='family_group',\
			store=True),
		'family_total_income':fields.function(_search_family_group, method=True, type='integer', string='Total Income',\
			multi='family_group'),
		'family_percapita_income':fields.function(_search_family_group, method=True, type='integer', string='Percapita Income',\
			multi='family_group'),
		'family_dwelling':fields.function(_search_family_group, method=True, type='text', string='Beneficiary Dwelling',\
			multi='family_group'),
		'family_income_description':fields.function(_search_family_group, method=True, type='text', string='Income', multi='family_group'),
		'family_num_men':fields.function(_search_family_group, method=True, type='integer', string='Men', multi='family_group'),
		'family_num_women':fields.function(_search_family_group, method=True, type='integer', string='Women', multi='family_group'),
		'family_num_children':fields.function(_search_family_group, method=True, type='integer', string='Children', multi='family_group'),
		'family_total_num':fields.function(_search_family_group, method=True, type='integer', string='Total', multi='family_group'),
		'member_id':fields.many2one('social.family.group.member', 'Search Beneficiary', required=True, readonly=False,\
			domain="[('member_folio','=',family_id)]"),
		'member_full_name':fields.function(_search_family_member, method=True, type='char', size=32, string='Full Name', multi='member',\
			store=True),
		'member_address':fields.function(_search_family_member, method=True, type='char', size=60, string='Address', multi='member'),
		'member_phone':fields.function(_search_family_member, method=True, type='char', size=25, string='Phone Number', multi='member'),
		'member_health':fields.function(_search_family_member, method=True, type='text', string='Beneficiary Health', multi='member'),
		'member_situation_eva':fields.function(_search_family_member, method=True, type='text',\
			string='Situation and Professional Evaluation', multi='member'),
		'member_age':fields.function(_search_family_member, method=True, type='integer', string='Age', multi='member'),
		'caseworker_name':fields.many2one('res.users', 'Caseworker Name', required=True, readonly=True),
		'date':fields.date('Social Report Date', required=True, readonly=False),
		'ben_family_members':fields.many2many('social.family.group.member', 'voucher_member_rel', 'voucher_code','member_rut',\
			'Family Members', domain="[('member_folio','=',family_id),('id','!=',member_id)]"),
		'ben_description':fields.text('Description of the Help', required=False, readonly=False),
		'ben_observations':fields.text('Observations of the Help', required=False, readonly=False),
		'state':fields.selection([\
			('1','In Demand - With Act'),\
			('2','In Demand - Without Act'),\
			('3','Delivered - With Act'),\
			('4','Delivered - Without Act')], 'Help State', required=True, readonly=False),
		'ben_requested_items':fields.one2many('social.help.voucher.item','table_id','Requested Items')
	}

	def state_delivered_without_act(self, cr, uid, ids, context=None):
		return self.write(cr, uid, ids, {'state':'4'})

	def state_delivered_with_act(self, cr, uid, ids, context=None):
		return self.write(cr, uid, ids, {'state':'3'})

	def state_in_demand_without_act(self, cr, uid, ids, context=None):
		return self.write(cr, uid, ids, {'state':'2'})
		
	def state_in_demand_with_act(self, cr, uid, ids, context=None):
		return self.write(cr, uid, ids, {'state':'1'})

	def _date(self, cr, uid, ids, context=None):
		today = datetime.date.today().strftime("%Y-%m-%d")
		return today
	
	def complete_folio_data(self, cr, uid, ids, folio, rut, context=None):
		send = []
		if folio and rut:
			obj_group = self.pool.get('social.family.group').browse(cr, uid, folio, context=context)
			obj_member = self.pool.get('social.family.group.member').browse(cr, uid, rut, context=context)

			return {
				'value': {
					'family_jjvv': obj_group.family_jjvv,
					'family_score': obj_group.family_score,
					'family_sector': obj_group.family_sector.name,
					'family_total_income': obj_group.family_total_income,
					'family_percapita_income': obj_group.family_percapita_income,
					'family_dwelling': obj_group.family_dwelling,
					'family_income_description': obj_group.family_income_description,
					'family_num_men': obj_group.family_num_men,
					'family_num_women': obj_group.family_num_women,
					'family_num_children': obj_group.family_num_children,
					'family_total_num': obj_group.family_total_num,
					'member_full_name': obj_member.member_full_name,
					'member_rut': obj_member.member_rut,
					'member_address': obj_member.member_address,
					'member_phone': obj_member.member_phone,
					'member_health': obj_member.member_health,
					'member_situation_eva': obj_member.member_situation_eva,
		                        'member_age': obj_member.member_age
				}
			}
		else:
			return {
				'value': {}
			}

	def create(self, cr, user, vals, context=None):
		sector = self.pool.get('social.family.group').browse(cr, user, vals['family_id'], context=context)
		count = self.pool.get('social.help.voucher').search(cr, user, [('family_sector','=',sector.family_sector.name)],count=True)+1
		self.pool.get('social.sector').write(cr, user, sector.family_sector.id, {'count': str(count)})

		count = self.pool.get('social.help.voucher').search(cr, user, [('caseworker_name','=',user)],count=True)+1
		self.pool.get('res.users').write(cr, user, user, {'count': str(count)})

		res = super(social_help_voucher, self).create(cr, user, vals, context=context)
		return res	

	def unlink(self, cr, user, ids, context=None):
		sectorDel = []
		usersDel = []
		for id in ids:
			sectorDel.append(self.pool.get('social.help.voucher').browse(cr, user, id, context=context).family_id.family_sector)
			usersDel.append(self.pool.get('social.help.voucher').browse(cr, user, id, context=context).caseworker_name)
	
		for sector in sectorDel:
			sector_aux = int(sector.count)-1
			if sector_aux < 0:
				self.pool.get('social.sector').write(cr, user, sector.id, {'count': '0'})
			else:
				self.pool.get('social.sector').write(cr, user, sector.id, {'count': str(sector_aux)})

		for users in usersDel:
			users_aux = int(users.count)-1
			if users_aux < 0:
				self.pool.get('res.users').write(cr, user, users.id, {'count': '0'})
			else:
				self.pool.get('res.users').write(cr, user, users.id, {'count': str(users_aux)})

		
		res = super(social_help_voucher, self).unlink(cr, user, ids, context=context)
		return res

	def _value(self, cr, uid, ids, context=None):
		#count = self.pool.get('social.help.voucher').search(cr, uid, [('id','>=','0')],count=True)
		ide = self.pool.get('social.help.voucher').search(cr, uid, [('id','>=','0')])
		if ide:
			count = self.pool.get('social.help.voucher').browse(cr, uid, ide[len(ide)-1]).voucher_code
			return count + 1
		else:   
			return 1

	_defaults = {
		'caseworker_name': lambda self,cr,uid,c: uid,
		'company_id': lambda self,cr,uid,c: self.pool.get('res.company')._company_default_get(cr, uid, 'social.help.voucher', context=c), 
		'state': '1',
		'date': _date,
		'voucher_code': _value,
	}
social_help_voucher()

